import os.path
import pathlib
import shlex
import shutil
import subprocess

import yada.config


def get_default_repo(yada_home=yada.config.get_yada_home()):
    return Repo("dot", yada_home=yada_home)


class Module():
    def __init__(self, repo, name):
        self.repo = repo
        self.name = name

    def __str__(self):
        return self.name

    @property
    def path(self):
        return self.repo.path / "modules" / self.name

    @property
    def files_path(self):
        return self.path / "files"

    def create(self):
        self.files_path.mkdir(parents=True, exist_ok=True)

    def import_file(self, path):
        if not path.is_absolute():
            path = pathlib.Path.cwd() / path
        destination = self.files_path / path.relative_to(yada.config.get_home())

        return (
            Operation(lambda: destination.parent.mkdir(parents=True, exist_ok=True),
                      "mkdir -p {dst}".format(dst=destination),
                      interactive=False),
            Operation(lambda: shutil.copy(path, destination),
                      "cp {src} {dst}".format(src=shlex.quote(str(path)),
                                              dst=shlex.quote(str(destination))))
        )

    def install(self):
        for f in self.files_path.glob("**/*"):
            destination = pathlib.Path("~").expanduser() / f.relative_to(self.files_path)
            if f.is_dir():
                yield Operation(lambda: destination.mkdir(parents=True, exist_ok=True),
                                "mkdir -p {}".format(destination),
                                interactive=True)
            else:
                command = "ln -rsf {src} {dst}".format(
                    src=shlex.quote(str(f)),
                    dst=shlex.quote(str(destination)))
                def backup_and_link():
                    if destination.is_file():
                        shutil.copy(destination,
                                    destination.with_suffix(destination.suffix + '.bkp'))
                    subprocess.call(shlex.split(command))
                yield Operation(backup_and_link,
                                command,
                                interactive=True)


class Repo():
    def __init__(self, name, yada_home=yada.config.get_yada_home()):
        self.name = name
        self.yada_home = pathlib.Path(yada_home)

    def __str__(self):
        return self.name

    @property
    def path(self):
        return (self.yada_home / self.name)

    def path_relative_to(self, path):
        common_parent = os.path.commonpath([path, self.path])
        return pathlib.Path(
            os.path.join(*[".." for _ in range(
                len(str(path).split(os.path.sep)) - len(common_parent.split(os.path.sep)))])) \
            / self.yada_home.relative_to(common_parent) \
            / self.name

    def module(self, module_name):
        return Module(self, module_name)


class Operation():
    def __init__(self, f, command, interactive=True):
        self.f = f
        self.command = command
        self.interactive = interactive

    def execute(self):
        self.f()
